library(data.table)
library(ggplot2)
library(gridExtra)
options(stringsAsFactors = FALSE)

args = commandArgs(trailingOnly=T)

homedir="/net/borenstein/vol1/PROJECTS/METABOLOMICS_CN/code_inprog/"
run_dir = args[1]
prefix = args[2]
load(paste0(homedir, run_dir, '/', prefix,"_out.rda"))
pos1 = dim(node_data[PValS < 0.05 & QValS < 0.1])[1]
pos2 = dim(node_data[PValS < 0.01 & QValS < 0.01])[1]
neg1 = dim(node_data[PValN < 0.05 & QValN < 0.1])[1]
neg2 = dim(node_data[PValN < 0.01 & QValN < 0.01])[1]

files = list.files(path=paste0(homedir,run_dir),pattern=paste0(prefix,"[0-9]*_shuff_network.txt"))

shuffs = data.table()
for(j in 1:length(files)){
      foo = fread(paste0(homedir, run_dir, "/",files[j]))
      shuffs = rbind(shuffs, foo)
      foo[,pos:=ifelse(V3 <= 0.01, 1,0)]
      foo[,neg:=ifelse(V3 <= 0.01, 1,0)]
      met_preds = rbind(met_preds, foo)

}
setnames(shuffs, c("total_pos1", "total_neg1", "total_pos2", "total_neg2"))
print(dim(shuffs[total_pos1 >= pos1])[1]/dim(shuffs)[1])
print(dim(shuffs[total_neg1 >= neg1])[1]/dim(shuffs)[1])
print(dim(shuffs[total_pos2 >= pos2])[1]/dim(shuffs)[1])
print(dim(shuffs[total_neg2 >= neg2])[1]/dim(shuffs)[1])


signif_counts = met_preds[,list(sum(pos)/length(files), sum(neg)/length(files)),by=V1]
setnames(signif_counts, c("compound","posCounts","negCounts"))
signif_counts = merge(signif_counts, node_data[,list(compound, TruePos,TrueNeg)], by="compound", all=T)
write.table(signif_counts, file = paste0(homedir, run_dir, "/", prefix, "_shuff_signif_by_met.txt"), quote=F, row.names=F, sep = "\t")

x0=ggplot(shuffs, aes(x=total_pos1))+ geom_histogram(binwidth=1) + geom_vline(xintercept=pos1, col="red", linetype=2) + theme_bw() + xlim(min(c(pos1, shuffs[,total_pos1])), max(c(pos1,shuffs[,total_pos1])))
x1=ggplot(shuffs, aes(x=total_pos2))+ geom_histogram(binwidth=1) + geom_vline(xintercept=pos2, col="red", linetype=2) + theme_bw() + xlim(min(c(pos2, shuffs[,total_pos2])), max(c(pos2,shuffs[,total_pos2])))

y0=ggplot(shuffs, aes(x=total_neg1))+ geom_histogram(binwidth=1) + geom_vline(xintercept=neg1, col="red", linetype=2) + theme_bw()+ xlim(min(c(neg1, shuffs[,total_neg1])), max(c(neg1,shuffs[,total_neg1])))
y1=ggplot(shuffs, aes(x=total_neg2))+ geom_histogram(binwidth=1) + geom_vline(xintercept=neg2, col="red", linetype=2) + theme_bw()+ xlim(min(c(neg2, shuffs[,total_neg2])), max(c(neg2,shuffs[,total_neg2])))

z=arrangeGrob(x0,x1,y0,y1, ncol=2)
ggsave(z, file=paste0(homedir, run_dir, '/',prefix, "_net_shuff_results.pdf"),width=8, height=8)

write.table(shuffs, file=paste0(homedir, run_dir, '/', prefix, "_shuff_network.txt"), quote=F, row.names=F, sep = "\t")
